==> style_selector-1.0.0-alpha1.tar.gz: style_selector/src/Services/FieldDiscovery.php <==
<?php

namespace Drupal\style_selector\Services;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\Display\EntityViewDisplayInterface;
use Drupal\layout_builder\Entity\LayoutEntityDisplayInterface;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Class FieldDiscovery
 *
 * Discover fields using Style Selector formatters.
 */
class FieldDiscovery {

  /**
   * The config factory object.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Config name.
   *
   * @var string
   */
  protected const CONFIG_NAME = 'style_selector.settings';

  /**
   * Field type
   *
   * @var string
   */
  protected const FIELD_TYPES = [
    'style_selector_css_class' => 'class',
    'style_selector_css_color' => 'color',
  ];

  public function __construct(ConfigFactoryInterface $config_factory)
  {
    $this->configFactory = $config_factory;
  }

  /**
   * Discover fields using the specified formatter.
   *
   * @param EntityViewDisplayInterface $display
   *
   * @param EntityInterface $entity
   *
   * @param array $formatter_types
   *
   * @return array
   *  An associative array of field names with additional field info:
   *  - settings: Array of settings configured in the formatter.
   *  - type: The field type ('class' or 'color').
   *  - classes: Array of module-defined classes for the field type.
   */
  public function getFields(EntityViewDisplayInterface $display, EntityInterface $entity, array $formatter_types) {
    // Get extra classes from module config.
    $config = $this->configFactory->get(self::CONFIG_NAME);
    $extra_classes = [
      'class' => $config->get('extra_css_classes'),
      'color' => $config->get('extra_color_classes'),
    ];

    $fields =[];

    // Get fields configured by Layout Builder.
    if ($display instanceof LayoutEntityDisplayInterface && $display->isLayoutBuilderEnabled()) {
      // Get entity-specific sections when the layout is customized.
      if ($display->isOverridable()
        && $entity->hasField('layout_builder__layout')
      ) {
        $sections = $entity->get('layout_builder__layout')->getSections();
      }
      // Get default sections otherwise.
      if (empty($sections)) {
        $sections = $display->getSections();
      }

      foreach ($sections as $section) {
        foreach ($section->getComponents() as $component) {
          $config = $component->get('configuration');

          // Only fields that use the specified formatter.
          if (isset($config['id']) && isset($config['formatter']['type'])) {
            $formatter = $config['formatter'];
            if (in_array($formatter['type'], $formatter_types)) {
              // Extract field name from the component ID.
              [, , , $name] = explode(':', $config['id']);
              if (!empty($name)) {
                // Only include Style Selector CSS Class field type.
                if ($type = $this->getStyleSelectorFieldType($name, $entity)) {
                  // $fields[$name]['settings'] ??= [];
                  $fields[$name] = [
                    'settings' => array_merge($fields[$name]['settings'] ?? [], $formatter['settings'] ?? []),
                    'type' => $type,
                    'classes' => $extra_classes[$type],
                  ];
                }
              }
            }
          }
        }
      }
    }

    // Get fields from standard manage display form.
    else {
      foreach ($display->getComponents() as $name => $component) {
        if (isset($component['type']) && in_array($component['type'], $formatter_types)) {
          // Only include Style Selector CSS Class field type.
          if ($type = $this->getStyleSelectorFieldType($name, $entity)) {
            // $fields[$name]['settings'] ??= [];
            $fields[$name] = [
              'settings' => array_merge($fields[$name]['settings'] ?? [], $component['settings'] ?? []),
              'type' => $type,
              'classes' => $extra_classes[$type],
            ];
          }
        }
      }
    }

    return $fields;
  }

  /**
   * Determine if a field is one of the Style Selector custom types.
   *
   * @param string $field_name
   *
   * @param Drupal\Core\Entity\EntityInterface $entity
   *
   * @return mixed
   *  Returns a short name for the field type if the field is a Style
   *  Selector field type. FALSE if it is not.
   */
  private function getStyleSelectorFieldType(string $field_name, EntityInterface $entity) {
    $field = ($entity->hasField($field_name)) ? $entity->get($field_name) : NULL;
    if ($field) {
      // return (in_array($field->getFieldDefinition()->getType(), self::FIELD_TYPES));
      return self::FIELD_TYPES[$field->getFieldDefinition()->getType()] ?? FALSE;
    }
    return FALSE;
  }

}

==> style_selector-2.0.0-alpha1.tar.gz: style_selector/src/Services/FieldDiscovery.php <==
<?php

namespace Drupal\style_selector\Services;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\Display\EntityViewDisplayInterface;
use Drupal\layout_builder\Entity\LayoutEntityDisplayInterface;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Class FieldDiscovery.
 *
 * Discover fields using Style Selector formatters.
 */
class FieldDiscovery {

  /**
   * The config factory object.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Config name.
   *
   * @var string
   */
  protected const CONFIG_NAME = 'style_selector.settings';

  /**
   * Field type.
   *
   * @var string
   */
  protected const FIELD_TYPES = [
    'style_selector_css_class' => 'class',
    'style_selector_css_color' => 'color',
  ];

  /**
   * The constructor.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * Discover fields using the specified formatter.
   *
   * @param \Drupal\Core\Entity\Display\EntityViewDisplayInterface $display
   *   An entity view display object.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Entity object.
   * @param array $formatter_types
   *   An array of formatter IDs to searchr for.
   *
   * @return array
   *   An associative array of field names with additional field info:
   *   - settings: Array of settings configured in the formatter.
   *   - type: The field type ('class' or 'color').
   *   - classes: Array of module-defined classes for the field type.
   */
  public function getFields(EntityViewDisplayInterface $display, EntityInterface $entity, array $formatter_types) {
    // Get extra classes from module config.
    $config = $this->configFactory->get(self::CONFIG_NAME);
    $extra_classes = [
      'class' => $config->get('extra_css_classes'),
      'color' => $config->get('extra_color_classes'),
    ];

    $fields = [];

    // Get fields configured by Layout Builder.
    if ($display instanceof LayoutEntityDisplayInterface && $display->isLayoutBuilderEnabled()) {
      // Get entity-specific sections when the layout is customized.
      if ($display->isOverridable()
        && $entity->hasField('layout_builder__layout')
      ) {
        $sections = $entity->get('layout_builder__layout')->getSections();
      }
      // Get default sections otherwise.
      if (empty($sections)) {
        $sections = $display->getSections();
      }

      foreach ($sections as $section) {
        foreach ($section->getComponents() as $component) {
          $config = $component->get('configuration');

          // Only fields that use the specified formatter.
          if (isset($config['id']) && isset($config['formatter']['type'])) {
            $formatter = $config['formatter'];
            if (in_array($formatter['type'], $formatter_types)) {
              // Extract field name from the component ID.
              [, , , $name] = explode(':', $config['id']);
              if (!empty($name)) {
                // Only include Style Selector CSS Class field type.
                if ($type = $this->getStyleSelectorFieldType($name, $entity)) {
                  // $fields[$name]['settings'] ??= [];
                  $fields[$name] = [
                    'settings' => array_merge($fields[$name]['settings'] ?? [], $formatter['settings'] ?? []),
                    'type' => $type,
                    'classes' => $extra_classes[$type],
                  ];
                }
              }
            }
          }
        }
      }
    }

    // Get fields from standard manage display form.
    else {
      foreach ($display->getComponents() as $name => $component) {
        if (isset($component['type']) && in_array($component['type'], $formatter_types)) {
          // Only include Style Selector CSS Class field type.
          if ($type = $this->getStyleSelectorFieldType($name, $entity)) {
            // $fields[$name]['settings'] ??= [];
            $fields[$name] = [
              'settings' => array_merge($fields[$name]['settings'] ?? [], $component['settings'] ?? []),
              'type' => $type,
              'classes' => $extra_classes[$type],
            ];
          }
        }
      }
    }

    return $fields;
  }

  /**
   * Get an array of values from a Drupal field value.
   *
   * EntityField::getValue() returns an indexed array of key, value pairs, e.g.:
   * [
   *  0: ['value': 'an actual value'],
   *  1: ['value': 'another actual value'],
   * ]
   * This method returns an array of those values. Empty if none.
   *
   * @param array $field_value
   *   Result of a call to EntityField::getValue().
   *
   * @return array
   *   An array of the actual values.
   */
  public function getFieldValues(array $field_value) {
    $values = [];
    foreach ($field_value as $value) {
      $values[] = $value['value'];
    }
    return $values;
  }

  /**
   * Determine if a field is one of the Style Selector custom types.
   *
   * @param string $field_name
   *   The name of the field to check for.
   * @param Drupal\Core\Entity\EntityInterface $entity
   *   The entity to check.
   *
   * @return mixed
   *   Returns a short name for the field type if the field is a Style
   *   Selector field type. FALSE if it is not.
   */
  private function getStyleSelectorFieldType(string $field_name, EntityInterface $entity) {
    $field = ($entity->hasField($field_name)) ? $entity->get($field_name) : NULL;
    if ($field) {
      return self::FIELD_TYPES[$field->getFieldDefinition()->getType()] ?? FALSE;
    }
    return FALSE;
  }

}
